% This code computes the likelihood for the encompassing model in Chang and Fernandez (2010)
% On the Sources of Aggregate Fluctuations in Emerging Economies. NBER WP
% 15938. April, 2010. It adapts a routine provided by Prof. John
% Landon-Lane at Rutgers University.

function loglike = LLcalc_constant3_me(y,A,H,F,Q,B,R)

% This function calculates the log likelihood of the following state-space
% system:
% y_t = A + H*s_t + u_t; Euu' = R, observation equation with measurement errors
% s_t = F*s_{t-1} + B*v_t;  system of transition equation where
% v_t is N(0,Q) donde Q deberia ser una matriz diagonal
%
%INPUTS:
%       y: nxT matrix of observables (y(i,t) is t^th observation of the
%          i^th observed variable
%       H: the observation matrix
%       F: the system kxk matrix; where k is the number of state vars.
%       B: the kxn matrix that selects the stochastic components for each state
%          variable from v_t.
%       Q: nxn covariance matrix of stochastic part (structural errors)
%       sinit: initial state vector (s(0|0))
%       Pinit: intitial covariance matrix of state vector P(0|0)
%       R: varcov of measurement errors of size nxn
%OUTPUTS:
%       loglike: log likelihood of State-Space system

% set up parameters so that the model fits into standard framework
n = size(F,1); %Number of variables in the state eq.
[m,T] = size(y);

Q1=B*chol(Q)';
sinit = zeros(n,1);
q = B*Q*B';  
Pinit = reshape(inv(eye(n^2) - kron(F,F))*vec(q),n,n); % Initialization as in Hamilton (1994)
       
[s,P,eta,f,loglike]=KF_constant(A,H,F,Q1,sinit,Pinit,y,R);

%--------------------------------------------------------------------------
function [s,P,eta,f,llf]=KF_constant(A,H,F,Q,s00,P00,y,R)
% This function computes the Kalman filter of a simple state space model
% It modifies Prof. Landon-Lane code to include the presence of a 
% exog. variables in the measurement eq.
% Output
% s: filtered states (m x 1 x T) % Note, he later uses m instead of m 
% P: P t|t    (m x m x T) %Note, this is the var-cov, that why it is mxm
% eta: forecast error (T x n)
% f: forecast error variance (n x n x T)

[n,T] = size(y); 
[n,m] = size(H); 
eta = zeros(T,n); 
s = zeros(m,1,T);
P = zeros(m,m,T);
lf = zeros(T,1);
l2p = log(2*pi);

%prediction (first observations)
sp=s00; 
Pp=P00; 
eta(1,:)=(y(:,1)-A-H*sp)'; 
f(:,:,1)=H*Pp*H' + R; 
%updating (first observations)
finv=f(:,:,1)\eye(n); 
K=Pp*H'*finv;
s(:,1,1)=sp+K*eta(1,:)'; 
P(:,:,1)=(eye(m)-K*H)*Pp; 

lf(1)=-(n/2)*l2p+0.5*log(det(finv))-0.5*eta(1,:)*finv*eta(1,:)';

for i=2:T
    sp=F*s(:,1,i-1);
    Pp=F*P(:,:,i-1)*F'+Q*Q';
    eta(i,:)=(y(:,i)-A-H*sp)';
    f(:,:,i)=H*Pp*H' + R;
    
    finv=f(:,:,i)\eye(n);
    K=Pp*H'*finv;
    s(:,1,i)=sp+K*eta(i,:)';
    P(:,:,i)=(eye(m)-K*H)*Pp;
    lf(i)=-(n/2)*l2p+0.5*log(det(finv))-0.5*eta(i,:)*finv*eta(i,:)';
end

% compute log likelihood function
llf=sum(lf);
