% This code solves and computes moments from the "encompassing model" in Chang and Fernandez (2010)
% On the Sources of Aggregate Fluctuations in Emerging Economies. NBER WP
% 15938. April, 2010. 

% Note: - The code uses the routines anal_deriv.m, gx_hx.m, num_eval.m, ir.m, mom.m provided by Stephanie Schmitt-Grohe and Martin Uribe.
%       - The user must have the symbolic toolbox to run the code.

clc
clear;

% Parameters to be estimated
sigma_g = 0.0109; % S.D. of growth shock.  
sigma_r = 0.0109; % S.D. of stationary foreign interest rate shock
sigma_z = 0.0041; % S.D. of stationary technology shock. 
rhog    = 0.72;   % AR(1) coef. of the growth process. 
rhor    = 0.83;   % AR(1) coef. of the foreign interest rate process.
rhoz    = 0.94;   % AR(1) coef. of the transitory technology process. 
mup     = 1.006;  % Productivity's long-run mean growth. 
phi     = 6.0;    % Investment Adj. Cost Parameter. 
eta     = 0.80;   % Elasticity of the spread to the Solow residual
teta    = 0.77;   % Fraction of the wage bill kept as working capital requirement

[fx,fxp,fy,fyp] = encompassing(rhog,rhor,rhoz,mup,phi,eta,teta);
[gx,hx] = gx_hx(fy,fx,fyp,fxp); 

% Comparing unconditional model-based and Mexican second moments 
% --------------------------------------------------------------
%Construct Variance/Covariance Matrix of Innovations to State Process
varshock = zeros(size(hx,1));
varshock(3,3) = sigma_g^2;
varshock(4,4) = sigma_r^2;
varshock(5,5) = sigma_z^2;

covar   = mom(gx,hx,varshock);
stds    = sqrt(diag(covar));
cov1    = mom(gx,hx,varshock,1);
MOMSM(1:4,1) = [100*stds(1), stds(2)/stds(1), stds(3)/stds(1), stds(4)/stds(1)]';
MOMSM(5:7,1) = [covar(1,2)/(stds(1)*stds(2)), covar(1,3)/(stds(1)*stds(3)), covar(1,4)/(stds(1)*stds(4))]';
MOMSM(8:9,1) = [covar(2,4)/(stds(2)*stds(4)), covar(3,4)/(stds(3)*stds(4))]';
MOMSM(10:13,1) = [cov1(1,1)/(stds(1)^2), cov1(2,2)/(stds(2)^2), cov1(3,3)/(stds(3)^2), cov1(4,4)/(stds(4)^2)]';

% Mexican Data based moments
load AGDataMEX %Loading Aguiar and Gopinath Mexican dataset
Y = data(1:end,6);  % Output
C = data(1:end,1);  % Consumption
Iv = data(1:end,2); % Investment
NX = data(1:end,5); % Trade Balance
TBY = NX./Y;        % Trade Balance Share

y = log(Y(2:end,1)) - log(Y(1:end-1,1));
c = log(C(2:end,1)) - log(C(1:end-1,1));
iv = log(Iv(2:end,1)) - log(Iv(1:end-1,1));
tby = TBY(2:end,1) - TBY(1:end-1,1);

DIFDATA = [ y, c, iv, tby ];
stdsd = std(DIFDATA);
MOMSD(1:4,1) = [100*stdsd(1), stdsd(2)/stdsd(1), stdsd(3)/stdsd(1), stdsd(4)/stdsd(1)]';
MOMSD(5:7,1) = [corr(y,c), corr(y,iv), corr(y,tby)]';
MOMSD(8:9,1) = [corr(tby,c), corr(tby,iv)]';
MOMSD(10:13,1) = [corr(y(2:end,1),y(1:end-1,1)), corr(c(2:end,1),c(1:end-1,1)), corr(iv(2:end,1),iv(1:end-1,1)), corr(tby(2:end,1),tby(1:end-1,1))]';

MOMS = [MOMSD, MOMSM]; %Table with observed and model-based moments

%Impulse responses after a 1 S.D. shock
%---------------------------------------

NumSimul = 40;

% Transitory Shock
x0_Tra = zeros(size(hx,1),1);
x0_Tra(5) = sigma_z; 
IR_Tra = 100*ir(gx,hx,x0_Tra,NumSimul);

t=(0:size(IR_Tra,1)-1)';
gy_Tr = IR_Tra(:,1);
gc_Tr = IR_Tra(:,2);
gi_Tr = IR_Tra(:,3);
y_Tr  = IR_Tra(:,10);
c_Tr  = IR_Tra(:,7);
i_Tr  = IR_Tra(:,9);
r_Tr  = IR_Tra(:,5);
h_Tr  = IR_Tra(:,8);
nx_Tr = IR_Tra(:,11);
w_Tr  = IR_Tra(:,14);
u_Tr  = IR_Tra(:,15);
s_Tr  = IR_Tra(:,16);

figure (1) % Figure plotting IRFs after a transitory tech. shock
subplot(3,4,1)
plot(t,gy_Tr)
title('gY')
legend('z Shock')

subplot(3,4,2)
plot(t,gc_Tr)
title('gC')

subplot(3,4,3)
plot(t,gi_Tr)
title('gI')

subplot(3,4,4)
plot(t,y_Tr)
title('Y')

subplot(3,4,5)
plot(t,c_Tr)
title('C')

subplot(3,4,6)
plot(t,i_Tr)
title('I')

subplot(3,4,6)
plot(t,i_Tr)
title('I')

subplot(3,4,7)
plot(t,r_Tr)
title('R')

subplot(3,4,8)
plot(t,h_Tr)
title('h')

subplot(3,4,9)
plot(t,nx_Tr)
title('TB/Y')

subplot(3,4,10)
plot(t,w_Tr)
title('W')

subplot(3,4,11)
plot(t,u_Tr)
title('u')

subplot(3,4,12)
plot(t,s_Tr)
title('S')

% R* Shock
x0_R = zeros(size(hx,1),1);
x0_R(4) = sigma_r; 
IR_R = 100*ir(gx,hx,x0_R,NumSimul);

t=(0:size(IR_R,1)-1)';
gy_R = IR_R(:,1);
gc_R = IR_R(:,2);
gi_R = IR_R(:,3);
y_R  = IR_R(:,10);
c_R  = IR_R(:,7);
i_R  = IR_R(:,9);
r_R  = IR_R(:,5);
h_R  = IR_R(:,8);
nx_R = IR_R(:,11);
w_R  = IR_R(:,14);
u_R  = IR_R(:,15);
s_R  = IR_R(:,16);

figure (2) % Figure plotting IRFs after a transitory tech. shock
subplot(3,4,1)
plot(t,gy_R)
title('gY')
legend('R* Shock')

subplot(3,4,2)
plot(t,gc_R)
title('gC')

subplot(3,4,3)
plot(t,gi_R)
title('gI')

subplot(3,4,4)
plot(t,y_R)
title('Y')

subplot(3,4,5)
plot(t,c_R)
title('C')

subplot(3,4,6)
plot(t,i_R)
title('I')

subplot(3,4,7)
plot(t,r_R)
title('R')

subplot(3,4,8)
plot(t,h_R)
title('h')

subplot(3,4,9)
plot(t,nx_R)
title('TB/Y')

subplot(3,4,10)
plot(t,w_R)
title('W')

subplot(3,4,11)
plot(t,u_R)
title('u')

subplot(3,4,12)
plot(t,s_R)
title('S')


% g Shock
x0_g = zeros(size(hx,1),1);
x0_g(3) = sigma_g; 
IR_g = 100*ir(gx,hx,x0_g,NumSimul);

t=(0:size(IR_g,1)-1)';
gy_g = IR_g(:,1);
gc_g = IR_g(:,2);
gi_g = IR_g(:,3);
y_g = IR_g(:,10);
c_g = IR_g(:,7);
i_g = IR_g(:,9);
r_g  = IR_g(:,5);
h_g  = IR_g(:,8);
nx_g = IR_g(:,11);
w_g  = IR_g(:,14);
u_g  = IR_g(:,15);
s_g  = IR_g(:,16);

figure (3) % Figure plotting IRFs after a transitory tech. shock
subplot(3,4,1)
plot(t,gy_g)
title('gY')
legend('g Shock')

subplot(3,4,2)
plot(t,gc_g)
title('gC')

subplot(3,4,3)
plot(t,gi_g)
title('gI')

subplot(3,4,4)
plot(t,y_g)
title('Y')

subplot(3,4,5)
plot(t,c_g)
title('C')

subplot(3,4,6)
plot(t,i_g)
title('I')

subplot(3,4,7)
plot(t,r_g)
title('R')

subplot(3,4,8)
plot(t,h_g)
title('h')

subplot(3,4,9)
plot(t,nx_g)
title('TB/Y')

subplot(3,4,10)
plot(t,w_g)
title('W')

subplot(3,4,11)
plot(t,u_g)
title('u')

subplot(3,4,12)
plot(t,s_g)
title('S')
